@@ -35,14 +35,18 @@
 #include <dirent.h>
 
 #include <glib.h>
-#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <gconf/gconf-client.h>
 #include "file-utils.h"
 #include "utf8-fnmatch.h"
+#include "main.h"
 
 
+#ifndef HAVE_MKDTEMP
+#include "mkdtemp.h"
+#endif                                        
+                                            
 #define BUF_SIZE 4096
 #define MAX_PATTERNS 128
 
@@ -1139,3 +1143,30 @@ check_permissions (const char *path,
 
 	return TRUE;
 }
+
+
+gboolean 
+is_program_in_path (const char *filename)
+{
+        char *str;
+	char *value;
+	int   result = FALSE;
+
+	value = g_hash_table_lookup (programs_cache, filename);
+	if (value != NULL) {
+		result = (strcmp (value, "1") == 0);
+		return result;
+	}
+
+        str = g_find_program_in_path (filename);
+        if (str != NULL) {
+                g_free (str);
+		result = TRUE;
+        }
+
+	g_hash_table_insert (programs_cache, 
+			     g_strdup (filename), 
+			     result ? "1" : "0");
+	
+        return result;
+}
