@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/**
+/*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -193,7 +193,9 @@ egg_recent_model_write_raw (EggRecentModel *model, FILE *file,
 	if (fputs (content, file) == EOF)
 		return FALSE;
 
+#ifndef G_OS_WIN32
 	fsync (fd);
+#endif
 	rewind (file);
 
 	return TRUE;
@@ -643,6 +645,8 @@ egg_recent_model_monitor_list (EggRecentModel *model, GList *list)
 static gboolean
 egg_recent_model_changed_timeout (EggRecentModel *model)
 {
+	model->priv->changed_timeout = 0;
+
 	egg_recent_model_changed (model);
 
 	return FALSE;
@@ -677,12 +681,17 @@ static void
 egg_recent_model_monitor (EggRecentModel *model, gboolean should_monitor)
 {
 	if (should_monitor && model->priv->monitor == NULL) {
+		char *uri;
+
+		uri = gnome_vfs_get_uri_from_local_path (model->priv->path);
 
 		gnome_vfs_monitor_add (&model->priv->monitor,
-					     model->priv->path,
-					     GNOME_VFS_MONITOR_FILE,
-					     egg_recent_model_monitor_cb,
-					     model);
+				       uri,
+				       GNOME_VFS_MONITOR_FILE,
+				       egg_recent_model_monitor_cb,
+				       model);
+
+		g_free (uri);
 
 		/* if the above fails, don't worry about it.
 		 * local notifications will still happen
@@ -857,14 +866,17 @@ static FILE *
 egg_recent_model_open_file (EggRecentModel *model)
 {
 	FILE *file;
+	mode_t prev_umask;
 	
 	file = fopen (model->priv->path, "r+");
 	if (file == NULL) {
 		/* be paranoid */
-		umask (077);
+		prev_umask = umask (077);
 
 		file = fopen (model->priv->path, "w+");
 
+		umask (prev_umask);
+
 		g_return_val_if_fail (file != NULL, NULL);
 	}
 
@@ -874,23 +886,54 @@ egg_recent_model_open_file (EggRecentModel *model)
 static gboolean
 egg_recent_model_lock_file (FILE *file)
 {
+#ifdef F_TLOCK
 	int fd;
+	gint	try = 5;
 
 	rewind (file);
 	fd = fileno (file);
 
-	return lockf (fd, F_LOCK, 0) == 0 ? TRUE : FALSE;
+	/* Attempt to lock the file 5 times,
+	 * waiting a random interval (< 1 second) 
+	 * in between attempts.
+	 * We should really be doing asynchronous
+	 * locking, but requires substantially larger
+	 * changes.
+	 */
+	
+	while (try > 0)
+	{
+		int rand_interval;
+
+		if (lockf (fd, F_TLOCK, 0) == 0)
+			return TRUE;
+
+		rand_interval = 1 + (int) (10.0 * rand()/(RAND_MAX + 1.0));
+			 
+           	g_usleep (100000 * rand_interval);
+
+		--try;
+	}
+
+	return FALSE;
+#else
+	return TRUE;
+#endif
 }
 
 static gboolean
 egg_recent_model_unlock_file (FILE *file)
 {
+#ifdef F_TLOCK
 	int fd;
 
 	rewind (file);
 	fd = fileno (file);
 
-	return lockf (fd, F_ULOCK, 0) < 0 ? FALSE : TRUE;
+	return (lockf (fd, F_ULOCK, 0) == 0) ? TRUE : FALSE;
+#else
+	return TRUE;
+#endif
 }
 
 static void
@@ -898,6 +941,10 @@ egg_recent_model_finalize (GObject *object)
 {
 	EggRecentModel *model = EGG_RECENT_MODEL (object);
 
+	if (model->priv->changed_timeout > 0) {
+		g_source_remove (model->priv->changed_timeout);
+	}
+
 	egg_recent_model_monitor (model, FALSE);
 
 
@@ -1260,6 +1307,7 @@ egg_recent_model_add_full (EggRecentModel * model, EggRecentItem *item)
 		ret = TRUE;
 	} else {
 		g_warning ("Failed to lock:  %s", strerror (errno));
+		fclose (file);
 		return FALSE;
 	}
 
